@@ -241,7 +241,9 @@ class FEField(Field):
         -----
         Assumes one cell type for the whole region!
         """
-        shape = parse_shape(shape, region.domain.shape.dim)
+        field_dim = region.field_dim if hasattr(region, 'field_dim')\
+            else region.domain.shape.dim
+        shape = parse_shape(shape, field_dim)
         Struct.__init__(self, name=name, dtype=dtype, shape=shape,
                         region=region)
         self.domain = self.region.domain
@@ -251,8 +253,7 @@ class FEField(Field):
         self._setup_kind()
         self._setup_shape()
 
-        self.surface_data = {}
-        self.point_data = {}
+        self.extra_data = {}
         self.ori = None
         self._create_interpolant()
         self._setup_global_base()
@@ -393,14 +394,13 @@ class FEField(Field):
         n_dof = region.vertices.shape[0]
 
         # Remap vertex node connectivity to field-local numbering.
-        conn, gel = self.domain.get_conn(ret_gel=True)
         if self.is_surface:
-            faces = gel.get_surface_entities()
-            aux = FESurface('aux', region, faces, conn)
+            aux = FESurface.from_region('aux', region)
             self.econn[:, :aux.n_fp] = aux.leconn
-            self.surface_data[region.name] = aux
+            self.extra_data[f'sd_{region.name}'] = aux
         else:
-            self.econn[:, :conn.shape[1]] = nm.take(remap, conn[region.cells])
+            conn = self.domain.get_conn(tdim=region.tdim, cells=region.cells)
+            self.econn[:, :conn.shape[1]] = nm.take(remap, conn)
 
         return n_dof, remap
 
@@ -526,18 +526,19 @@ class FEField(Field):
         """
         region = self.domain.regions[region_name]
         shape = region.shape
-        dim = region.dim
+        dim = region.field_dim if hasattr(region, 'field_dim') else region.dim
 
         if integration is None:
             integration == region.kind
 
         if 'facet' in integration:
-            if region_name not in self.surface_data:
+            name = f'sd_{region_name}'
+            if name not in self.extra_data:
                 reg = self.domain.regions[region_name]
                 self.domain.create_surface_group(reg)
                 self.setup_surface_data(reg, None)
 
-            sd = self.surface_data[region_name]
+            sd = self.extra_data[name]
 
             # This works also for surface fields.
             key = sd.face_type
@@ -549,6 +550,10 @@ class FEField(Field):
             else:
                 data_shape = (sd.n_fa, n_qp, dim, sd.n_fp)
 
+        elif (integration == 'cell' and self.region.tdim > 1 and
+              region.tdim == 1):
+            data_shape = (shape.n_cell, 0, dim, 2)  # bar elements
+
         elif integration in ('cell', 'custom'):
             _, weights = integral.get_qp(self.gel.name)
             n_qp = weights.shape[0]
@@ -742,7 +747,7 @@ class FEField(Field):
     def create_bqp(self, region_name, integral):
         gel = self.gel
 
-        sd = self.surface_data[region_name]
+        sd = self.extra_data[f'sd_{region_name}']
         bqpkey = (integral.order, sd.bkey)
         if bqpkey not in self.qp_coors:
             qp = self.get_qp(sd.face_type, integral)
@@ -947,7 +952,11 @@ class FEField(Field):
 
             conns = [conn]
             mat_ids = [self.cmesh.cell_groups]
-            descs = mesh.descs[:1]
+            tdim = self.cmesh.tdim
+            descs = f'{tdim}_{conn.shape[1]}'
+            if descs not in mesh.descs:
+                msg = f'element type {descs} not in mesh! ({mesh.descs})'
+                raise ValueError(msg)
 
             if extra_nodes:
                 coors = self.coors
@@ -955,8 +964,8 @@ class FEField(Field):
             else:
                 coors = self.coors[:self.n_vertex_dof]
 
-            mesh = Mesh.from_data(self.name, coors, None, conns,
-                                  mat_ids, descs)
+            mesh = Mesh.from_data(self.name, coors[:, :tdim], None, conns,
+                                  mat_ids, [descs])
 
         return mesh
 
@@ -1067,19 +1076,39 @@ class FEField(Field):
         else:
             return self.coors[nods]
 
-    def get_connectivity(self, region, integration, trace_region=None):
-        """
-        Convenience alias to `Field.get_econn()`, that is used in some terms.
-        """
-        return self.get_econn(integration, region, trace_region=trace_region)
-
     def get_econn(self, conn_type, region, trace_region=None, local=False):
         """
         Get extended connectivity of the given type in the given region.
-        """
-        ct = conn_type.type if isinstance(conn_type, Struct) else conn_type
 
-        if ct in ('cell', 'custom'):
+        Parameters
+        ----------
+        conn_type: tuple or string
+            DOF connectivity type, eg. ('cell', 3) or 'cell'.
+            If the topological dimension not specified, it is taken from
+            region.tdim.
+        region: sfepy.discrete.common.region.Region
+            The region for which the connectivity is required.
+        trace_region: None or string
+            If not None, return mirror connectivity according to `local`.
+        local: bool
+            If True, return local connectivity w.r.t. facet nodes,
+            otherwise return global connectivity w.r.t. all mesh nodes.
+
+        Returns
+        -------
+        econn: numpy.ndarray
+            The extended connectivity array.
+        """
+        if isinstance(conn_type, tuple):
+            integration, tdim = conn_type
+        else:
+            integration, tdim = conn_type, region.tdim
+
+        if integration == 'cell' and tdim == 1 and self.region.tdim > 1:
+            # bar elements
+            conn = self.extra_data[f'bars_{region.name}']
+
+        elif (integration in ('cell', 'custom')) and (trace_region is None):
             if region.name == self.region.name:
                 conn = self.econn
             else:
@@ -1088,66 +1117,80 @@ class FEField(Field):
                 ii = self.region.get_cell_indices(cells, true_cells_only=tco)
                 conn = nm.take(self.econn, ii, axis=0)
 
-        elif ct == 'facet':
-            if region.name not in self.surface_data:
+        elif integration == 'cell' and trace_region is not None:
+            name = f'sd_{region.name}'
+            sd = self.extra_data[name]  # FEPhantomSurface
+            conn = sd.get_connectivity(local=local, trace_region=trace_region)
+
+        elif integration == 'facet':
+            name = f'sd_{region.name}'
+            if name not in self.extra_data:
                 self.domain.create_surface_group(region)
                 self.setup_surface_data(region)
 
             if self.is_surface:
                 local = True
-            sd = self.surface_data[region.name]
+            sd = self.extra_data[name]
             conn = sd.get_connectivity(local=local, trace_region=trace_region)
 
-        elif ct == 'point':
-            conn = self.point_data[region.name]
+        elif integration == 'point':
+            conn = self.extra_data[f'pd_{region.name}']
 
         else:
             raise NotImplementedError('connectivity type %s' % ct)
 
         return conn
 
-    def setup_extra_data(self, geometry, info):
-        dct = info.dc_type.type
+    def setup_extra_data(self, info):
+        for dct, tdim in set(info.dof_conn_types.values()):
+            if dct == 'facet':
+                reg = info.get_region()
+                mreg_name = info.get_region_name(can_trace=False)
+                mreg_name = None if reg.name == mreg_name else mreg_name
+                self.domain.create_surface_group(reg)
+                self.setup_surface_data(reg, mreg_name)
 
-        if dct == 'facet':
-            reg = info.get_region()
-            mreg_name = info.get_region_name(can_trace=False)
-            mreg_name = None if reg.name == mreg_name else mreg_name
-            self.domain.create_surface_group(reg)
-            self.setup_surface_data(reg, mreg_name)
+            elif dct == 'edge':
+                raise NotImplementedError('dof connectivity type %s' % dct)
 
-        elif dct == 'edge':
-            raise NotImplementedError('dof connectivity type %s' % dct)
+            elif dct == 'point':
+                self.setup_point_data(self, info.region)
 
-        elif dct == 'point':
-            self.setup_point_data(self, info.region)
+            elif dct == 'cell' and tdim == 1 and self.region.tdim > 1:
+                # bar elements
+                self.setup_bar_data(self, info.region)
 
-        elif dct not in ('cell', 'custom'):
-            raise ValueError('unknown dof connectivity type! (%s)' % dct)
+            elif dct not in ('cell', 'custom'):
+                raise ValueError('unknown dof connectivity type! (%s)' % dct)
 
     def setup_surface_data(self, region, trace_region=None):
         """nodes[leconn] == econn"""
         """nodes are sorted by node number -> same order as region.vertices"""
-        if region.name not in self.surface_data:
-            name = 'surface_data_%s' % region.name
+        name = f'sd_{region.name}'
+        if name not in self.extra_data:
             if trace_region is not None and region.tdim == (region.dim - 1):
                 sd = FEPhantomSurface(name, region, self.econn)
             else:
                 sd = FESurface(name, region, self.efaces, self.econn,
                                self.region)
-            self.surface_data[region.name] = sd
+            self.extra_data[name] = sd
 
-        if region.name in self.surface_data and trace_region is not None:
-            sd = self.surface_data[region.name]
+        if name in self.extra_data and trace_region is not None:
+            sd = self.extra_data[name]
             sd.setup_mirror_connectivity(region, trace_region)
 
-        return self.surface_data[region.name]
-
     def setup_point_data(self, field, region):
-        if region.name not in self.point_data:
+        name = f'pd_{region.name}'
+        if name not in self.extra_data:
             conn = field.get_dofs_in_region(region, merge=True)
             conn.shape += (1,)
-            self.point_data[region.name] = conn
+            self.extra_data[name] = conn
+
+    def setup_bar_data(self, field, region):
+        name = f'bars_{region.name}'
+        if name not in self.extra_data:
+            conn = region.domain.get_conn(tdim=1)[region.cells]
+            self.extra_data[name] = conn
 
     def create_mapping(self, region, integral, integration,
                        return_mapping=True):
@@ -1166,7 +1209,6 @@ class FEField(Field):
         """
         domain = self.domain
         coors = domain.get_mesh_coors(actual=True)
-        dconn = domain.get_conn()
 
         iels = region.get_cells(true_cells_only=(region.kind == 'cell'))
         transform = (self.basis_transform[iels] if self.basis_transform
@@ -1179,8 +1221,8 @@ class FEField(Field):
             qp = self.get_qp('v', integral)
             bf = self.get_base('v', 0, integral, iels=iels)
 
-            conn = nm.take(dconn, iels.astype(nm.int32), axis=0)
-            mapping = FEMapping(coors, conn, poly_space=geo_ps)
+            dconn = domain.get_conn(tdim=region.tdim, cells=iels)
+            mapping = FEMapping(coors, dconn, poly_space=geo_ps)
             out = mapping.get_mapping(qp.vals, qp.weights, bf, poly_space=ps,
                                       ori=self.ori, transform=transform)
 
@@ -1198,8 +1240,11 @@ class FEField(Field):
                 raise ValueError(msg)
 
             sd = domain.surface_groups[region.name]
-            esd = self.surface_data[region.name]
+            esd = self.extra_data[f'sd_{region.name}']
 
+            face_indices = region.get_facet_indices()
+            cells = face_indices[:, 0]
+            dconn = domain.get_conn(tdim=region.tdim, cells=cells)
             conn = sd.get_connectivity()
             mapping = FEMapping(coors, conn, poly_space=geo_ps)
 
@@ -1220,7 +1265,8 @@ class FEField(Field):
                     se_bf_bg = se_bf_bg[sd.fis[:, 1]]
                     se_ebf_bg = self.get_base(esd.bkey, 1, integral)
                     se_ebf_bg = se_ebf_bg[sd.fis[:, 1]]
-                    se_conn = dconn[sd.fis[:, 0], :]
+                    remap = prepare_remap(cells, cells.max() + 1)
+                    se_conn = dconn[remap[sd.fis[:, 0]], :]
                 else:
                     se_bf_bg, se_ebf_bg, se_conn = None, None, None
 
