@@ -266,7 +266,8 @@ class Problem(Struct):
 
             if conf is None:
                 self.conf = Struct(options={}, ics={},
-                                   ebcs={}, epbcs={}, lcbcs={}, materials={})
+                                   ebcs={}, epbcs={}, lcbcs={}, materials={},
+                                   solvers={})
 
         self.equations = equations
         self.fields = fields
@@ -1002,14 +1003,6 @@ class Problem(Struct):
         self.domain.save_regions_as_groups(filename,
                                            region_names=region_names)
 
-    def save_field_meshes(self, filename_trunk):
-
-        output('saving field meshes...')
-        for field in self.fields:
-            output(field.name)
-            field.write_mesh(filename_trunk + '_%s')
-        output('...done')
-
     def get_evaluator(self, reuse=False):
         """
         Either create a new Evaluator instance (reuse == False),
@@ -1039,9 +1032,10 @@ class Problem(Struct):
         epbc_indx = []
         for ii, variable in enumerate(variables.iter_state(ordered=True)):
             eq_map = variable.eq_map
-            ebc_indx.append(eq_map.eq_ebc + variables.di.ptr[ii])
-            epbc_indx.append((eq_map.master + variables.di.ptr[ii],
-                              eq_map.slave + variables.di.ptr[ii]))
+            offset = variables.di.indx[variable.name].start
+            ebc_indx.append(eq_map.eq_ebc + offset)
+            epbc_indx.append((eq_map.master + offset, eq_map.slave + offset))
+
         ebc_indx = nm.concatenate(ebc_indx)
         epbc_indx = nm.concatenate(epbc_indx, axis=1)
         return ebc_indx, epbc_indx
@@ -1345,7 +1339,7 @@ class Problem(Struct):
 
         def poststep_fun(ts, vec):
             variables = self.equations.variables
-            # EBCs need to be applied here because of algrbraically computed
+            # EBCs need to be applied here because of algebraically computed
             # variables in elastodynamics solvers.
             variables.set_state(vec, self.active_only, apply_ebc=True)
             if step_hook is not None:
