@@ -103,6 +103,12 @@ def parse_options(opts, separator=':'):
     return out
 
 
+def make_title(filenames):
+    title = ', '.join(filenames)
+    title = title if len(title) < 80 else title[:77] + '...'
+    return title
+
+
 def make_cells_from_conn(conns, convert_to_vtk_type):
     cells, cell_type, offset = [], [], []
     _offset = 0
@@ -143,8 +149,12 @@ def make_grid_from_mesh(mesh, add_mat_id=False):
     cells, cell_type, offset = make_cells_from_conn(
         {desc: mesh.get_conn(desc)}, vtk_cell_types,
     )
+    try:
+        grid = pv.UnstructuredGrid(offset, cells, cell_type, points)
+
+    except TypeError: # Pyvista >= 0.39.0
+        grid = pv.UnstructuredGrid(cells, cell_type, points)
 
-    grid = pv.UnstructuredGrid(offset, cells, cell_type, points)
     if add_mat_id:
         add_mat_id_to_grid(grid, mesh.cmesh.cell_groups)
 
@@ -322,7 +332,7 @@ def pv_plot(filenames, options, plotter=None, step=None,
     color = None
 
     if plotter is None:
-        plotter = pv.Plotter()
+        plotter = pv.Plotter(title=make_title(filenames))
 
     if step_inc is not None:
         plotter.clear()
@@ -416,6 +426,7 @@ def pv_plot(filenames, options, plotter=None, step=None,
                                      use_cache=use_cache)
 
         pipe = [steps[fstep].copy()]
+        pos_bnds = pipe[0].bounds
 
         if field in fields_map:  # subregion
             mat_val = fields_map[field]
@@ -445,7 +456,8 @@ def pv_plot(filenames, options, plotter=None, step=None,
         if isinstance(factor, tuple):
             ws = nm.diff(nm.reshape(pipe[-1].bounds, (-1, 2)), axis=1)
             size = ws[ws > 0.0].min()
-            fmax = nm.abs(pipe[-1][field]).max()
+            factor_field = warp if warp is not None else field
+            fmax = nm.abs(pipe[-1][factor_field]).max()
             factor = 0.01 * float(factor[1]) * size / fmax
 
         if warp:
@@ -467,9 +479,9 @@ def pv_plot(filenames, options, plotter=None, step=None,
             plot_info.append('warp=%s, factor=%.2e' % (warp, factor))
 
         position = opts.get('p', 0)  # determine plotting slot
-        bnds = pipe[-1].bounds
         if 'p' in opts:
-            size = nm.array(bnds[1::2]) - nm.array(bnds[::2])
+            size = nm.array(pos_bnds[1::2]) - nm.array(pos_bnds[::2])
+            size = nm.maximum(size, 1e-2 * size.max())
             pipe.append(pipe[-1].copy())
             pos1 = position % options.max_plots
             pos2 = position // options.max_plots
@@ -833,7 +845,8 @@ def main():
     options = parser.parse_args()
 
     pv.set_plot_theme("document")
-    plotter = pv.Plotter(off_screen=options.off_screen)
+    plotter = pv.Plotter(off_screen=options.off_screen,
+                         title=make_title(options.filenames))
 
     if options.anim_output_file:
         _, n_steps = read_mesh(options.filenames, ret_n_steps=True)
