@@ -353,10 +353,10 @@ class ScipyIterative(LinearSolver):
 
         solver_kwargs.update(prec_args)
         if conf.method == 'gmres':
+            from pkg_resources import parse_version
             import scipy as sp
-            from distutils.version import LooseVersion
 
-            if LooseVersion(sp.__version__) >= '1.4.0':
+            if parse_version(sp.__version__) >= parse_version('1.4.0'):
                 solver_kwargs.update({'callback_type' : 'legacy'})
 
         try:
@@ -805,12 +805,11 @@ class MUMPSSolver(LinearSolver):
     def __init__(self, conf, **kwargs):
         import sfepy.solvers.ls_mumps as mumps
 
-        if not mumps.use_mpi:
-            raise AttributeError('No mpi4py found! Required by MUMPS solver.')
-
         LinearSolver.__init__(self, conf, mumps=mumps, mumps_ls=None,
                               mumps_presolved=False, **kwargs)
         mumps.load_mumps_libraries()  # try to load MUMPS libraries
+        if not mumps.use_mpi:
+            raise AttributeError('No mpi4py found! Required by MUMPS solver.')
 
         self.clear()
 
@@ -822,7 +821,8 @@ class MUMPSSolver(LinearSolver):
 
         self.presolve(mtx, use_mtx_digest=conf.use_mtx_digest)
 
-        out = rhs.copy()
+        out = rhs.copy(order='F')
+
         self.mumps_ls.set_rhs(out)
         self.mumps_ls(3)  # solve
 
@@ -969,7 +969,9 @@ class SchurMumps(MUMPSSolver):
             schur_list.append(nm.arange(slc.start, slc.stop, slc.step, dtype='i'))
 
         self.mumps_ls.set_mtx_centralized(mtx)
-        out = rhs.copy()
+
+        out = rhs.copy(order='F')
+
         self.mumps_ls.set_rhs(out)
 
         S, y2 = self.mumps_ls.get_schur(nm.hstack(schur_list))
