@@ -1,6 +1,8 @@
 """
 Time stepping solvers.
 """
+from inspect import signature
+from functools import partial
 import numpy as nm
 
 from sfepy.base.base import (get_default, output, assert_,
@@ -367,9 +369,13 @@ def transform_equations_ed(equations, materials):
     dvname = 'd' + vname
     ddvname = 'dd' + vname
     new_variables = variables.copy()
+    if new_variables[uname]._order is None:
+        raise ValueError('state variable orders have to be specified when using'
+                         ' auto_transform_equations!')
+    num = len(new_variables.state)
     new_variables.extend([
-        FieldVariable(duname, 'unknown', variables[uname].field),
-        FieldVariable(dduname, 'unknown', variables[uname].field),
+        FieldVariable(duname, 'unknown', variables[uname].field, order=num),
+        FieldVariable(dduname, 'unknown', variables[uname].field, order=num + 1),
         FieldVariable(dvname, 'test', variables[uname].field,
                       primary_var_name=duname),
         FieldVariable(ddvname, 'test', variables[uname].field,
@@ -434,8 +440,8 @@ def gen_multi_vec_packing(di, names, extra_variables=False):
     iu = di.indx[names['u']]
     iv = di.indx[names['du']]
     ia = di.indx[names['ddu']]
-    ie = slice(iu.stop, di.ptr[-3])
-    iue = slice(0, di.ptr[-3])
+    ie = slice(iu.stop, iv.start)
+    iue = slice(0, iv.start)
     assert_(iu.stop == di.n_dof[names['u']])
     assert_(iv.start == ie.stop)
     assert_(ia.start == iv.stop)
@@ -461,7 +467,7 @@ def gen_multi_vec_packing(di, names, extra_variables=False):
         return nm.concatenate(args)
 
     indices = dict(indices=(iue, iu, ie, iv, ia),
-                   n_dof=di.ptr[-1], n_uedof=ie.stop, n_arg=n_arg)
+                   n_dof=di.n_dof_total, n_uedof=ie.stop, n_arg=n_arg)
     unpack.__dict__.update(indices)
     pack.__dict__.update(indices)
     return unpack, pack
@@ -586,6 +592,8 @@ class ElastodynamicsBaseTS(TimeSteppingSolver):
             self.di, self.conf.var_names,
             extra_variables=self.get('extra_variables', False),
         )
+        self.unpack = unpack
+        self.pack = pack
 
         ts = self.ts
         vec0 = init_fun(ts, vec0)
@@ -617,13 +625,23 @@ class ElastodynamicsBaseTS(TimeSteppingSolver):
 
         return vec, unpack, pack
 
-    def clear_lin_solver(self):
+    def clear_lin_solver(self, clear_constant_matrices=True):
         self.nls.lin_solver.clear()
         self.matrix = None
+        if clear_constant_matrices:
+            self.constant_matrices = None
 
     @standard_ts_call
     def __call__(self, vec0=None, nls=None, init_fun=None, prestep_fun=None,
                  poststep_fun=None, status=None, **kwargs):
+        sig = signature(init_fun)
+        if len(sig.parameters) == 3:
+            init_fun = partial(init_fun, self)
+
+        sig = signature(poststep_fun)
+        if len(sig.parameters) == 3:
+            poststep_fun = partial(poststep_fun, self)
+
         vec, unpack, pack = self.get_initial_vec(
             nls, vec0, init_fun, prestep_fun, poststep_fun)
 
@@ -655,7 +673,7 @@ class ElastodynamicsBaseTS(TimeSteppingSolver):
                 output('dt:', ts.dt, 'new dt:', new_dt, 'status:', status,
                        verbose=self.verbose)
                 if new_dt != ts.dt:
-                    self.clear_lin_solver()
+                    self.clear_lin_solver(clear_constant_matrices=False)
 
                 if status.result == 'accept':
                     break
@@ -1216,11 +1234,12 @@ class BatheTS(ElastodynamicsBaseTS):
         nlst.lin_solver = self.ls2
         return nlst
 
-    def clear_lin_solver(self):
-        self.nls.lin_solver.clear()
+    def clear_lin_solver(self, clear_constant_matrices=True):
+        ElastodynamicsBaseTS.clear_lin_solver(
+            self, clear_constant_matrices=clear_constant_matrices,
+        )
         self.ls1 = self.ls2 = None
         self.matrix1 = None
-        self.matrix = None
 
     def step(self, ts, vec, nls, pack, unpack, prestep_fun):
         """
