@@ -22,6 +22,7 @@ examples = [
     'diffusion/poisson_field_dependent_material.py',
     'diffusion/poisson_functions.py',
     'diffusion/poisson_neumann.py',
+    'diffusion/poisson_nonlinear_material.py',
     'diffusion/poisson_periodic_boundary_condition.py',
     'diffusion/sinbc.py',
     'diffusion/time_advection_diffusion.py',
@@ -80,6 +81,18 @@ else:
         'navier_stokes/navier_stokes2d_iga.py',
     ])
 
+try:
+    import primme
+
+except ImportError:
+    pass
+
+else:
+    examples.extend([
+        'linear_elasticity/modal_analysis_declarative.py',
+    ])
+
+
 @pytest.mark.parametrize('ex_filename', examples)
 def test_examples(ex_filename, output_dir):
     conditions = run_declaratice_example(
