@@ -10,6 +10,11 @@ import sfepy.base.testing as tst
 filename_meshes = [data_dir + '/meshes/elements/%s_2.mesh' % geom
                    for geom in ['1_2', '2_3', '2_4', '3_4', '3_8', '3_2_4']]
 
+not_tested_terms = ['dw_ns_dot_grad_s',
+                    'dw_lin_spring', 'dw_lin_truss', 'ev_lin_truss_force',
+                    'dw_tl_he_ogden_ad']
+
+
 def make_term_args(arg_shapes, arg_kinds, arg_types, ats_mode, domain,
                    material_value=None, poly_space_base=None):
     from sfepy.base.base import basestr
@@ -159,7 +164,7 @@ def make_term_args(arg_shapes, arg_kinds, arg_types, ats_mode, domain,
 
         else:
             str_args.append('user%d' % ii)
-            args[str_args[-1]] = None
+            args[str_args[-1]] = sh if callable(sh) else None
 
     materials = Materials(materials)
     variables = Variables(variables)
@@ -320,7 +325,7 @@ def test_term_call_modes(data):
         for _, term_cls in ordered_iteritems(term_table):
             if (domain_geometry not in term_cls.geometries) \
                or ("dg" in term_cls.name) \
-               or (term_cls.name == "dw_ns_dot_grad_s"):
+               or (term_cls.name in not_tested_terms):
                 continue
 
             rnames = []
